#*
This file is the template for generating header files that contain definitions
for the syslog logging module that map log statement to 32-bit identifiers that
are included in the log statement instead of textual descriptions.
*#
\#ifndef $name

/*
 * Header file automatically generated by pymomo.syslog.LogDefinitionMap from 
 * the following source files:
#for $src in $sources
 * $src
#end for
 * 
 * These definitions are for logging data in the hardware system log included with
 * the 16-bit momo controller firmware.  Each log entry is tagged with a unique 
 * 32-bit value that corresponds to a definition in the log definition files associated 
 * with the source code for that module.  You can use the momo SystemLog tool to parse
 * and read the systemlog entries generated using these identifiers.
 */

#for $msg,$val in $messages.iteritems()
\#define $msg 			$val
#end for

\#endif
