#basic tests to ensure that everything is installed and working properly

def test_required_packages():
	"""
	Make sure that the required dependencies of pymomo are installed
	"""

	import decorator
	#TODO add the rest of the packages

def test_import_pymomo():
	"""
	Make sure pymomo is accessible for import
	"""

	import pymomo